package bugs.toplink.serializable;

import static org.junit.Assert.assertTrue;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;

import org.junit.Test;

public class UpdateTest {

	@Test
	public void testUpdateEventWithSerializable()	{
		EntityManagerFactory emf = Persistence.createEntityManagerFactory("model");
		Employee e = new Employee();
		final boolean[] listenerCalled = new boolean[1];
		Employee.setListener(new Runnable() {
			public void run() {
				listenerCalled[0] = true;
			}
		});

		EntityManager em = emf.createEntityManager();
		em.getTransaction().begin();
		e = em.merge(e);
		em.getTransaction().commit();
		em.close();
		
		/* 
		 * If we change the properties field on an entity retrieved from a 
		 * EntityManager#find method call, methods with @PostUpdate are called. 
		 */
		em = emf.createEntityManager();
		em.getTransaction().begin();
		e = em.find(Employee.class, e.getId());
		e.putProperty("key1", "value1");
		e = em.merge(e);
		em.getTransaction().commit();
		em.close();
		assertTrue("Update event not issued 1", listenerCalled[0]);
		
		/* Reset the flag, for the next update */
		listenerCalled[0] = false;

		/* Store a copy of the properties to verify that they are indeed different */
		Map<String, Serializable> initialProperties = new HashMap<String, Serializable>(e.getProperties());
		
		/* 
		 * If we change the properties field on a detached entity and then
		 * call EntityManager#merge, methods with @PostUpdate are not called.
		 */
		em = emf.createEntityManager();
		em.getTransaction().begin();
		e.putProperty("key2", "value2");
		e = em.merge(e);
		em.getTransaction().commit();
		em.close();
		
		/* 
		 * Verify that the properties did change and as such an event should
		 * be issued
		 */
		assertTrue("e properties were not changed", 
				!e.getProperties().equals(initialProperties));

		assertTrue("Update event not issued 2", listenerCalled[0]);
		
		emf.close();
	}
}
